# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base
  include UserInfo

  helper :layout, :nueva_edicion
  before_filter :objeto_seccion, :xls_request, :tiempo_comienzo, :por_pagina, :sesion_timeout, :autorizar, :configurar_idioma, :except => [:entrada, :identificacion]

  # helper :all # include all helpers, all the time
  #protect_from_forgery # See ActionController::RequestForgeryProtection for details
  layout "layout", :except => ["entrada"] #Siempre incluimos el mismo layout general menos en identificacion

  # Comprobacion de inactividad. Configurado a 30 minutos.
  def sesion_timeout
    session[:timeout] ||= Time.now
    if Time.now - session[:timeout] > 3600
      session[:usuario_identificado_id] = nil
      session[:timeout] = Time.now
      mensaje = _("Tiempo de inactividad en el sistema superado. Vuelva a identificarse para acceder.")
      if request.xhr?
        # Si la peticion es ajax
        render (:update) {|page|  page.mensaje_informacion params[:update], mensaje, :tipo_mensaje => "mensajefallo"} if params[:update]
        render (:update) {|page|  page.mensaje_informacion params[:update_listado], mensaje, :tipo_mensaje => "mensajefallo"} if params[:update_listado]
      else
        # Si la peticion no es ajax
        flash[:mensaje] = mensaje
        redirect_to  :seccion => :administracion, :controller => :usuario, :action => :entrada      
      end
    end
    session[:timeout] = Time.now
  end

  # Cada peticion se mira el idioma de la session del usuario y se configura el locale.
  def configurar_idioma
    if session[:idioma_seleccionado].nil? or session[:idioma_seleccionado] == ""
      #set_locale "es_ES"
      FastGettext.locale = I18n.locale = 'es_ES'
    else
      #set_locale session[:idioma_seleccionado]
      FastGettext.locale = I18n.locale = session[:idioma_seleccionado] 
    end
  end 

  def por_pagina
   session[:por_pagina] = params[:por_pagina] if params[:por_pagina]
  end

  # Metodo general para representar al final del layout cuando se esta en modo development el tiempo total de una peticion.
  def tiempo_comienzo     
    @tiempo_comienzo = Time.now
  end

  # Scrub sensitive parameters from your log
  # filter_parameter_logging :password

  # Este metodo general esta aqui para que los controladores puedan crear objetos fechas desde parametros del date_select
  # FIXME: Parece algo tonto pero no consigo resolverlo de otra forma
  def fecha parametros
     return Date.new parametros["(1i)"].to_i, parametros["(2i)"].to_i, parametros["(3i)"].to_i
  end

  # Se comprueba si el acceso tiene un usuario identificado, y se comprueba tambien que el usuario tiene permisos para acceder la seccion en la que se esta navegando.
  def autorizar
    #puts "------------------> ERROR!!!!! No existe la variable de sesion para usuario_identificado_id" unless session[:usuario_identificado_id]
    @usuario_identificado = Usuario.find_by_id session[:usuario_identificado_id]
    #puts "------------------> @usuario_identificado -> " + @usuario_identificado.inspect
    unless @usuario_identificado.class.name == "Usuario" 
      msg_error _("Debe identificarse para acceder al sistema.")
      redirect_to  :seccion => :administracion, :controller => :usuario, :action => :entrada
      return false
    end
    
     unless params[:seccion] == "inicio" or params[:seccion] == "info" or ( @usuario_identificado and @usuario_identificado.send( params[:seccion] ) == true )
       msg_error _( "no tiene los derechos para ir a la sección " ) + params[:seccion]
       redirect_to :seccion => :inicio, :controller => :tarea
       return false
     end

     UserInfo.current_user = @usuario_identificado 
  end

  def singularizar_seccion
    params[:seccion].gsub(/(.*)(s)/) {$1.gsub(/one/, "on")}
  end
 
  # Este metodo devuelve la entidad dentro de la seccion con la que estamos trabajando. 
  # En el caso de la seccion proyectos pone en @proyecto el proyecto con el que estamos trabajando
  # En el caso de la seccion agentes pone en @agente el agente con el que estamos trabajando
  def objeto_seccion
    case params[:seccion]
    when "proyectos" then
      @proyecto = Proyecto.find(params[:proyecto_id]) if params[:proyecto_id]
      @permitir_configuracion = @proyecto && ( @proyecto.definicion_estado.nil? || !@proyecto.definicion_estado.cerrado ) && params[:menu] == "configuracion"
      @permitir_formulacion = @proyecto && @proyecto.definicion_estado && @proyecto.definicion_estado.formulacion && params[:menu] == "formulacion"
      @permitir_ejecucion = @proyecto && @proyecto.definicion_estado && @proyecto.definicion_estado.aprobado && params[:menu] == "ejecucion"
    when "agentes"
      then @agente = Agente.find(params[:agente_id]) if params[:agente_id] 
    end
  end

  # Cambia el formato de respuesta cuando se pide un XLS
  def xls_request
    request.format = :xls if (params[:selector] && (params[:selector][:salida] == 'fichero' || params[:selector][:fichero] == '1')) || params[:fichero] == "1"
  end

  # Para usar en exportaciones a XLS
  def xls_filename
    params[:seccion] + "-" + params[:controller] + "-" + params[:action] + ".xls"
  end



  #--
  # MENSAJES
  #++

  def msg objeto, otros={}
    cadena = nil
    if objeto.class == String
      cadena = objeto.to_s
    elsif objeto.class == Array
      cadena = "<ul>" + objeto.inject("") {|suma, e | suma + "<li>" + e + "</li>"} + "</ul>"
    elsif objeto.methods.include?(:errors)
      if objeto.errors.empty?
        otros[:ok] = true
        cadena = _("Los datos se han guardado correctamente.") unless otros[:eliminar]
        cadena = _("Se ha eliminado correctamente.") if otros[:eliminar]
      else
        otros[:error] = true
        cadena = _("Se han producido errores.") + "<br>"
        objeto.errors.each {|a, m| cadena << m + "<br>" }
      end 
    end
    if cadena
      if otros[:now]
        flash.now[:mensaje_error] = cadena if otros[:error]
        flash.now[:mensaje_ok] = cadena if otros[:ok]
        flash.now[:mensaje] = cadena unless otros[:error] || otros[:ok]
      else
        flash[:mensaje_error] = cadena if otros[:error]
        flash[:mensaje_ok] = cadena if otros[:ok]
        flash[:mensaje] = cadena unless otros[:error] || otros[:ok]
      end
    end
  end

  # dibuja el mensaje de error o de exito
  def msg_error mensaje, otros={}
    otros[:error] = true 
    msg mensaje, otros 
  end

  # dibuja el mensaje de error o de exito al eliminar un objeto
  def msg_eliminar objeto
    msg objeto, :eliminar => true
  end


  # Conversiones entre formatos de moneda accesibles desde todos los controladores
  def float_a_moneda cadena
    ('%.2f' % numero).to_s.sub(".",",") if numero
  end

  def moneda_a_float cadena
    if cadena
      # primero comprobamos que el punto corresponde a miles ( formato => n.nnn )
      # y solo en ese caso lo eliminamos => Permitimos especificar decimales con punto
      numero = (cadena =~ /[\d]+\.[\d]{3}/) ? cadena.delete(".") : cadena
      # Si se esta usando la coma decimal, la cambiamos por un punto ( formato => n,nn o n,n )
      # REVISAR: Esto permite que haya coma para miles, pero impide que metan mas de 2 decimales con coma
      numero = numero.sub(/,/,".") if numero =~ /[\d]+,[\d]{1,2}$/
      return numero.to_f
    end
  end

end
