# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de la gestión de los documentos.
# Este controlador es utilizado desde las secciones:
# * Sección proyectos: se utiliza para listar, comentar, subir y descargar documentos
# * Sección financiaciones: se utiliza para listar, comentar, subir y descargar documentos
class DocumentoBusquedaController < ApplicationController


	# en proyectos y en financiación: se redirecciona por defecto a ordenado
  def index
    #session[:documento_filtro] = etiqueta_todos 
    redirect_to :action => "ordenado"
  end

#--
# FILTRADO DE DOCUMENTOS
#++

  # Hace un filtrado segun las etiquetas que tengamos seleccionadas
  def filtrado
    # Primero vaciamos la variable del filtros (hay una por seccion)
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] = []
    # Incluimos el valor del selector si hay algo seleccionado 
    session[("filtro_etiquetas_" + params[:seccion]).to_sym] << params[:selector][:etiqueta_nueva] if params[:selector][:etiqueta_nueva].to_i != 0
    # Vamos incluyendo las etiquetas seleccionadas previamente
    if params[:filtro_etiqueta]
      params[:filtro_etiqueta].each_value do |id|
        session[("filtro_etiquetas_" + params[:seccion]).to_sym] << id if id != "" && id.to_i > 0
      end 
    end
    redirect_to :action => :listado
  end

  # Incluye la etiqueta seleccionada para poder filtrar
  def anadir_filtro
    etiquetas_filtrado
    render :update do |page|
      page.replace_html params[:update], :partial => "elemento_filtrado", :locals => { :indice => params[:indice].to_i, :valor => params[:selector][:etiqueta_nueva], :ultimo => true }
    end
  end

  # Elimina la etiqueta seleccionada de las elegidas para filtrar
  def quitar_filtro
    render :update do |page|
      page.replace_html params[:update], :inline => ''
    end
  end

	# en proyectos y en financiación: establece los parametros de ordenación
  def ordenado
    session[:documento_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC" 
    session[:documento_orden] = params[:orden] ? params[:orden] : "adjunto_file_name" 
    redirect_to :action => "listado"
  end

	# en proyectos y en financiación: lista los documentos disponibles
  def listado
   # Establece las condiciones para la busqueda de documentos segun sea de proyecto o global
    condiciones_proyecto = ( @proyecto ? "documento.proyecto_id = " + @proyecto.id.to_s : "" )
   # Si tiene pacs (es un convenio) incluye en la busqueda los documentos de estos
    @proyecto.pacs.each do |pac|
      condiciones_proyecto += " OR documento.proyecto_id = " + pac.id.to_s
    end if @proyecto
    condiciones_proyecto = "( " + condiciones_proyecto + " )" if condiciones_proyecto
    # Hace la busqueda de documentos segun tengamos filtros o no
    if session[("filtro_etiquetas_" + params[:seccion]).to_sym] and session[("filtro_etiquetas_" + params[:seccion]).to_sym].length > 0
      condicion_filtros = (params[:seccion] == "proyectos" ? condiciones_proyecto + " AND " : "") + "etiqueta_x_documento.etiqueta_id IN (?)", session[("filtro_etiquetas_" + params[:seccion]).to_sym]
      @documentos = Documento.paginate( :include => ["etiqueta_x_documento" ], :conditions => condicion_filtros, :group => ["documento.id"], :having => ["count(documento.id) >= " + session[("filtro_etiquetas_" + params[:seccion]).to_sym].length.to_s ], :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => (session[:por_pagina] or 20) )
    else
      condicion_filtros = condiciones_proyecto if params[:seccion] == "proyectos"
      #condicion_filtros = "etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
      condicion_filtros = nil if condicion_filtros == "(  )" # Chapucilla por un error cuando las condiciones estan vacias

      @documentos =  Documento.paginate( :include => [ "etiqueta_x_documento", ], :conditions => condicion_filtros, :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => session[:por_pagina] )
    end
    etiquetas_filtrado
  end


  def listado_gastos
    joins =  "LEFT JOIN gasto_x_documento  ON gasto_x_documento.documento_id = documento.id LEFT 
              JOIN gasto ON gasto.id = gasto_x_documento.gasto_id LEFT JOIN gasto_x_proyecto ON gasto_x_proyecto.gasto_id = gasto.id "
    # Ponemos las condiciiones en funcion de estar en la seccion de agentes o proyectos.
    condiciones =  params[:seccion] == "proyectos" ? {"gasto_x_proyecto.proyecto_id" => @proyecto.id} : {"gasto.agente_id" => @agente.id}
    @documentos = Documento.paginate :joins => joins , :conditions => condiciones, :include => [:gasto],
                                     :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => session[:por_pagina]
    #etiquetas_filtrado
    render :action => "listado" 
  end

  def listado_transferencias
    joins =  "LEFT JOIN transferencia_x_documento ON transferencia_x_documento.documento_id = documento.id LEFT 
              JOIN transferencia ON transferencia.id = transferencia_x_documento.transferencia_id"
    # Ponemos las condiciiones en funcion de estar en la seccion de agentes o proyectos.
    condiciones = {"transferencia_x_documento.transferencia_id" => (@proyecto||@agente).transferencia}
    @documentos = Documento.paginate :conditions => condiciones, :include => [:transferencia_x_documento], :order => (session[:documento_orden] + " " + session[:documento_asc_desc] ), :page => params[:page], :per_page => session[:por_pagina]
    #etiquetas_filtrado
    render :action => "listado" 
  end


#--
# VISUALIZACIÖN DE DOCUMENTOS POR ESPACIOS
#++



    # Devuelve el conjunto de etiquetas valido para la seccion en la que estamos
    def etiquetas_filtrado
      condiciones_etiqueta = ""
      condiciones_etiqueta = "etiqueta.nombre LIKE ?", _("Exportacion") + " %" if params[:seccion] == "administracion"
      condiciones_etiqueta = "etiqueta.tipo = 'proyecto'" if params[:seccion] == "proyectos"
      @etiquetas = Etiqueta.find( :all, :conditions => condiciones_etiqueta ).collect{ |e| [e.nombre, e.id] }.sort{ |a, b| a[0] <=> b[0]}
    end




end
