# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado de los estados de proyecto y financiación. Este controlador es utilizado desde las secciones:
# * Sección proyectos: controla los estados
# * Sección financiaciones: controla los estados

class EstadoController < ApplicationController
  before_filter :verificar_usuario
  before_filter :verificar_coordinador, :only => [ :cambiar_estado, :modificar_estado, :regresar_estado_anterior ]

	# Verifica que el usuario esta autentificado y pertenece al proyecto
  def verificar_usuario
    if ! ( @usuario_identificado.send(params[:seccion]) == true and params[:seccion] == "proyectos" and @proyecto.usuario_x_proyecto.find_by_usuario_id(@usuario_identificado.id) )
      msg_error _("No tiene permisos suficientes para acceder a este proyecto.")
      redirect_to "/inicio/tarea/inicio"
    end
  end

	# Verifica que el usuario es un coordinador del proyecto
  def verificar_coordinador
    if ! ( params[:seccion] == "proyectos" and @proyecto.usuario_x_proyecto.find_by_usuario_id(@usuario_identificado.id).rol == "coordinador" )
      if request.xhr?
        render :partial => "derecho_insuficiente"
      else
        msg_error _("No tiene permisos suficientes para cambiar el estado del proyecto.")
        redirect_to :action => 'listado'
      end
    end
  end

	# en proyectos y en financiación: se redirecciona por defecto a listado
  def index
    redirect_to :action => 'listado'
  end

	# en proyectos y en financiación: lista los estados anteriores y actual
  def listado
    @estados =  @proyecto.estado 
    @campos = ['observacion']
  end

	# en proyectos y en financiación: prepara el formulario de cambio de estado actual
  def cambiar_estado
      @estado_actual = @proyecto.estado_actual 
      @definicion_estado_siguiente = @estado_actual ? @estado_actual.definicion_estado.estado_hijo : DefinicionEstado.find_all_by_primer_estado(true)
      @definicion_estado_siguiente.collect!{ |a| [a.nombre, a.id] }
      render :partial => "formulario_cambiar_estado"

  end

	# en proyectos y en financiación: modifica el estado actual y llama a Correo.deliver_cambio_estado
  def modificar_estado
    if comprobar_etiquetas
      @msg = ""
      if @proyecto.estado_actual
        @estado_actual = @proyecto.estado_actual
        @estado_actual.estado_actual = false
        observacion = @estado_actual.observacion = params[:estado][:observacion] if params[:estado][:observacion] and @estado_actual
        fecha = @estado_actual.fecha_fin = Time.now()
        @estado_actual.save
      end
      @estado = Estado.new
      @estado.proyecto_id = @proyecto.id 
      @estado.definicion_estado_id = params[:definicion_estado][:id]
      @estado.fecha_inicio = Time.now
      @estado.estado_actual = true
      @estado.usuario_id = @usuario_identificado.id
      @estado.save
      for usuario in @proyecto.usuario
        begin
          Correo.deliver_cambio_estado(usuario, params[:seccion], @proyecto, @estado)
        rescue
          msg_error _("Se han producido errores enviando correos a los usuarios")
        end
      end if @estado.errors.empty?
      asignar_tareas
    else
      message = _("No se puede cambiar de estado. Compruebe que tiene los documentos necesarios y que están adecuadamente etiquetados.") + " "
      message << _("Los documentos etiquetados necesarios para cambiar el estado actual son:") + "    "
      message << DefinicionEstado.find(params[:definicion_estado][:id]).etiqueta.inject("") {|todo, a| todo + '  "' + a.nombre + '"   ' }
      msg_error message
    end
    redirect_to :action => 'listado'
  end

  def comprobar_etiquetas
    if @proyecto.estado_actual
      todas_etiquetas = Array.new
      for documento in @proyecto.documento
        todas_etiquetas << documento.etiqueta
      end
      todas_etiquetas = todas_etiquetas.flatten
      validacion = true
      for etiqueta in  DefinicionEstado.find(params[:definicion_estado][:id]).etiqueta
        validacion = false unless todas_etiquetas.include?(etiqueta)
      end
      return validacion
    else
      return true
    end
  end

  def asignar_tareas
    usuario =  @proyecto.reload.usuario_x_proyecto.find_by_rol("coordinador").usuario
    for tarea in  @proyecto.estado_actual.definicion_estado.definicion_estado_tarea
      Tarea.create :titulo => tarea.titulo, :descripcion => tarea.descripcion, :tipo_tarea_id => tarea.tipo_tarea_id, 
                   :usuario_id => usuario.id, :proyecto_id => @proyecto.id, :estado_tarea_id  => tarea.estado_tarea_id
       begin
         Correo.deliver_cambio_tarea(usuario, params[:seccion], @proyecto.titulo, tarea.titulo, tarea.descripcion)
       rescue
         @msg << "no se ha podido mandar el mail a " + usuario.correoe + "<br>"
       end
    end
    
  end

  def regresar_estado_anterior
    @msg = ""
    @proyecto.estado_actual.destroy
    unless @proyecto.reload.estado.empty?
      ultimo_estado = (@proyecto.reload.estado.sort {|a,b| a.updated_at <=> b.updated_at}).last
      ultimo_estado.update_attribute("estado_actual", true)
    end
    for usuario in @proyecto.usuario
      begin
        Correo.deliver_cambio_estado(usuario, params[:seccion], @proyecto.reload, "Vuelta al estado previo")
      rescue
        @msg << "no se ha podido mandar el mail a " + usuario.correoe + "<br>"
      end
    end
    redirect_to :action => 'listado'
  end

	# en proyectos y en financiación: permite hacer observación al cambiar de estado
  def detalle
    @estado = Estado.find(params[:id])
    render :partial => "observaciones"
  end

	# en proyectos y en financiación: cierra el formulario de observación
  def cerrar_detalle
    render  :inline => ""
  end
end
#done
