# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# agente

class Agente < ActiveRecord::Base
  #untranslate_all
  belongs_to :moneda_principal, :class_name => "Moneda", :foreign_key => "moneda_id"
  belongs_to :moneda_intermedia, :class_name => "Moneda", :foreign_key => "moneda_intermedia_id"
  belongs_to :pais

  has_many :tarea, :order => "fecha_inicio"
  has_many :agente_x_moneda
  has_many :moneda, :through => :agente_x_moneda
  has_many :libro
  has_many :etapa, :dependent => :destroy, :order => "nombre"
  has_many :usuario_x_agente, :dependent => :destroy
  has_many :usuario, :through => :usuario_x_agente, :order => "nombre"
  has_many :gasto_x_agente, :dependent => :destroy
  has_many :transferencia_x_agente, :dependent => :destroy
  has_many :transferencia, :through => :transferencia_x_agente
  has_many :presupuesto_x_agente, :dependent => :destroy
  has_many :presupuesto
  has_many :proyecto_x_financiador, :dependent => :destroy
  has_many :proyecto_x_implementador, :dependent => :destroy
  has_many :proyecto_implementador, :through => :proyecto_x_implementador, :source => :proyecto
  has_many :proyecto_financiador, :through => :proyecto_x_financiador, :source => :proyecto
  has_many :subpartida
  has_one :espacio, :dependent => :nullify

  validate :nombre_mayusculas
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :pais_id, :message => _("País") + _(" no puede estar vacío."), :if => :implementador

  before_destroy :verificar_borrado
  after_save :cambiar_tasas_cambio, :actualiza_espacio
  after_destroy :verificar_borrado, :borrar_espacio

  def actualiza_espacio
    espacio_padre = Espacio.find_by_nombre_and_espacio_padre_id("Socias Locales", pais.espacio.id) if implementador && socia_local
    espacio_padre = Espacio.find_by_nombre_and_espacio_padre_id("Delegaciones", pais.espacio.id) if implementador && !socia_local
    tipo = "agente" if implementador
    espacio_padre = Espacio.find_by_nombre_and_espacio_padre_id("Financiadores",nil) if financiador && !implementador
    tipo = "financiador" if financiador && !implementador
    if espacio_padre
      if espacio
        self.espacio.update_attributes :nombre => nombre, :espacio_padre_id => espacio_padre.id
      else
        esp = Espacio.create :nombre => nombre, :agente_id => self.id, :espacio_padre_id => espacio_padre.id, :descripcion => _("Espacio raiz del " + tipo) + ": " + nombre
        puts "-----------> ERROR: " + esp.errors.inspect unless esp.errors.empty?
        self.espacio = esp if esp.errors.empty?
      end
      # Actualiza los espacios automaticos
      if implementador
        for esp in Espacio.find_all_by_definicion_espacio_agente(true)
          Espacio.find_or_create_by_nombre_and_espacio_padre_id_and_definicion_espacio_agente_id(esp.nombre,self.espacio.id,self.id)
        end
      end
      if financiador && !implementador
        for esp in Espacio.find_all_by_definicion_espacio_financiador(true)
          Espacio.find_or_create_by_nombre_and_espacio_padre_id_and_definicion_espacio_financiador_id(esp.nombre,self.espacio.id,self.id)
        end
      end      
    else
      logger.info "ERROR: No existe el espacio padre para el agente " + nombre
    end
  end  


  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

	# Devuelve los proyectos (evitando convenios) de los cuales es implementador y que estan abiertos
  def proyecto_implementador_abierto
    proyecto_implementador.collect{ |p| [p.nombre, p.id] if p.estado_actual && p.estado_actual.definicion_estado.aprobado && !p.convenio? } 
  end

	# Devuelve un hash con el arqueo de todos los libros del agente en la etapa indicada 
  def remanente etap
    remanentes = {} 
    if etap && etap.class.name == "Etapa" && etap.etapa_siguiente
      # Recorre todas las cuentas del agente
      self.libro.each do |l|
        remanentes[l.id] = l.arqueo([], etap.fecha_inicio, etap.fecha_fin)
      end 
    end
    return remanentes
  end

	# Genera transferencias en la etapa siguiente a la indicada con los remanentes de esta
  def generar_remanente etap
    errores = false
    # Primero borramos todas las transferencias de remanentes de la etapa siguiente para todos los libros
    self.libro.each do |l|
      Transferencia.all(:conditions => {:proyecto_id => nil, :fecha_recibido => etap.etapa_siguiente.fecha_inicio, :remanente => true}).each {|t| t.destroy}
    end
    # Y recorremos todos los remanentes (si los hay) para ir aplicandolos libro a libro
    remanentes = self.remanente etap
    if remanentes.count > 0
      remanentes.keys.each do |libro_id|
        total = remanentes[libro_id][:entrante] - remanentes[libro_id][:saliente]
        if total != 0
          t = Transferencia.new(	:remanente => true, :tipo => "remanente",
					:libro_destino_id => libro_id,
					:fecha_recibido => etap.etapa_siguiente.fecha_inicio,
					:importe_recibido => total, :importe_cambiado => total,
					:observaciones => _("Remanente") + " " + etap.nombre )
          t.save
          errores = true unless t.errors.empty?
          errors.add_to_base( _("Problemas generando transferencia de remanente para el libro") + ": " + self.libro.find_by_id(libro_id).nombre) unless t.errors.empty?
          logger.info "---------------> Error generando transferencia de remanente en agente " + t.errors.inspect unless t.errors.empty?
        end
      end
    end
    # Si no se han producido errores...
    unless errores
      # ... indica a la etapa que los saldos estan ok
      etap.saldos_transferidos = true
      etap.save
      # ... y a la etapa siguiente (si existe y esta cerrada) que tiene que revisar los suyos
      if (etap_siguiente = etap.etapa_siguiente) && etap_siguiente.cerrada
        etap_siguiente.saldos_transferidos = false
        etap_siguiente.save
      end
    end
  end

  private

  def cambiar_tasas_cambio
    # Para modificaciones de agente que incluyan moneda principal, actualiza las tasas de cambio
    self.etapa.each { |e| TasaCambio.cambia_moneda(e) } if id_was && moneda_id_changed?
  end

  def self.financiador_publico_y_privado
    return  find_by_nombre("OTRAS FINANCIACIONES PÚBLICAS"), find_by_nombre("OTRAS FINANCIACIONES PRIVADAS")
  end


  def verificar_borrado
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto.empty?
    errors.add( "presupuesto",_("hay presupuestos")) unless self.presupuesto_x_agente.empty?
    errors.add( "gasto", _("hay gastos")) unless self.gasto_x_agente.empty?
    errors.add( "libro", _("hay libros")) unless self.libro.empty?
    errors.add( "transferencia", _("hay transferencias")) unless self.transferencia_x_agente.empty?
    errors.add_to_base( _("Un agente tiene que estar vacio para poder ser borrado.") ) unless errors.empty?
    return false unless errors.empty?
  end

  def borrar_espacio
    if espacio
      if implementador
        for esp in Espacio.find_all_by_definicion_espacio_agente(true)
          esp = Espacio.find_by_espacio_padre_id_and_definicion_espacio_agente_id(espacio.id,esp.id)
          esp.destroy if esp
        end
      end
      if financiador && !implementador
        for esp in Espacio.find_all_by_definicion_espacio_financiador(true)
          esp = Espacio.find_by_espacio_padre_id_and_definicion_espacio_financiador_id(espacio.id,esp.id)
          esp.destroy if esp
        end
      end
      espacio.destroy
    end
  end
  
end
  

