# encoding: UTF-8
class AutomatizacionObserver < ActiveRecord::Observer
  include UserInfo
  observe :gasto, :presupuesto

  def after_save(objeto)
    # Esto tendremos que cambiarlo cuando mezclemos comentarios y marcado para evitar que haya anotaciones dobles (comentario + marcado => comentario)
    # Si ha cambiado y no ha sido solo el marcado...
    cambios = objeto.changed
    cambios.delete("marcado_id")
    if cambios.size > 0
      marcar_elemento objeto
      comentario_elemento objeto if current_user
    end
  end

  # Marca el elemento cuando se ha modificado o creado
  def marcar_elemento(objeto)
    # Marca el elemento solo si no hemos hecho cambios en el marcado
    if objeto.marcado_id == objeto.marcado_id_was
      marc=Marcado.find_by_automatico(true)
      if marc
        # Guarda evitando callbacks 
        self.update_column(:marcado_id, marc.id)
      end
    end
  end

  # Genera un comentario automatico con la modificacion
  def comentario_elemento(objeto)
    # Tenemos un problema con las traducciones aqui :(
    texto = objeto.class.name + " " + "creado" if objeto.new_record?
    texto = objeto.class.name + " " + "modificado" unless objeto.new_record?
    objeto.comentario << Comentario.create(:usuario_id => current_user.id, :texto => texto)
  end
end

