# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++

# ActiveResource que devuelve proyecto desde Alfresco.
class Proyecto < ActiveRecord::Base
  #untranslate_all
  has_many :proyecto_x_pais, :dependent => :destroy
  #has_many :pais, :through => :proyecto_x_pais, :order => "nombre"
  has_many :pais, :through => :proyecto_x_pais
  belongs_to :agente
  belongs_to :moneda_principal, :class_name => "Moneda", :foreign_key => "moneda_id"
  belongs_to :moneda_intermedia, :class_name => "Moneda", :foreign_key => "moneda_intermedia_id"
  has_one :datos_proyecto, :dependent => :destroy
  has_one :espacio, :dependent => :nullify
  has_many :usuario_x_proyecto, :dependent => :destroy
  has_many :gasto_x_proyecto, :dependent => :destroy
  has_many :gasto, :through => :gasto_x_proyecto
  has_many :usuario, :through => :usuario_x_proyecto, :order => "nombre"
  has_many :etapa, :dependent => :destroy, :order => "nombre"
  belongs_to :libro_principal, :class_name => "Libro", :foreign_key => "libro_id"
  has_many :libro_x_proyecto
  has_many :libro, :through => :libro_x_proyecto, :order => "nombre"
  has_many :transferencia
  has_many :presupuesto
  has_many :definicion_dato, :through => :proyecto_x_definicion_dato
  has_many :proyecto_x_definicion_dato, :dependent => :destroy
  has_many :dato_texto
  has_many :documento #, :through => :espacio
  has_many :objetivo_especifico, :order => "codigo"
  has_many :resultado, :order => "codigo"
  has_many :actividad, :order => "codigo"
  has_many :estado, :dependent => :destroy, :include => :definicion_estado
  has_one  :definicion_estado, :through => :estado_actual
  has_one  :estado_actual, :class_name => "Estado", :foreign_key => "proyecto_id", :conditions => ["estado.estado_actual"]
  has_many :tarea, :order => "fecha_inicio", :dependent => :destroy
  has_many :partida_financiacion, :order => "codigo"
  has_many :proyecto_x_moneda, :dependent => :destroy
  has_many :moneda, :through => :proyecto_x_moneda, :uniq => true
  belongs_to :pais_principal, :class_name => "Pais", :foreign_key => "pais_principal_id"

  has_many :proyecto_x_sector_poblacion, :dependent => :destroy
  has_many :sector_poblacion, :through => :proyecto_x_sector_poblacion, :source => :sector_poblacion, :order => "nombre"
  has_many :proyecto_x_area_actuacion, :dependent => :destroy
  has_many :area_actuacion, :through => :proyecto_x_area_actuacion, :source => :area_actuacion, :order => "nombre"
  has_many :proyecto_x_sector_intervencion, :dependent => :destroy
  has_many :sector_intervencion, :through => :proyecto_x_sector_intervencion, :source => :sector_intervencion, :order => "nombre"

  has_many :proyecto_x_financiador, :dependent => :destroy
  has_many :proyecto_x_implementador, :dependent => :destroy
  has_many :implementador, :through => :proyecto_x_implementador, :source => :agente, :order => "nombre"
  has_many :financiador, :through => :proyecto_x_financiador, :source => :agente, :order => "nombre"

  has_many :subpartida  

  # Para cofinanciacion de proyectos
  has_many :proyecto_x_proyecto, :dependent => :destroy
  has_many :proyecto_x_proyecto_cofinanciado, :foreign_key => 'proyecto_cofinanciador_id', :class_name => 'ProyectoXProyecto', :dependent => :destroy
  has_many :proyecto_cofinanciador, :through => :proyecto_x_proyecto, :class_name => 'Proyecto', :order => "nombre"
  has_many :proyecto_cofinanciado, :through => :proyecto_x_proyecto_cofinanciado, :class_name => 'Proyecto', :order => "nombre"

  # Para convenios
  belongs_to :convenio, :class_name => "Proyecto", :foreign_key => "convenio_id"
  has_many :pacs, :class_name => "Proyecto", :foreign_key => 'convenio_id', :order => "nombre", :dependent => :destroy

  validate :nombre_mayusculas 
  validates_uniqueness_of :nombre, :message => _("Nombre") +   _(" repetido.")
  validates_uniqueness_of :titulo, :message => _("Título") +   _(" repetido.")
  validates_presence_of :titulo, :message => _("Título") + _(" no puede estar vacío.")
  validates_presence_of :nombre, :message => _("Nombre") + _(" no puede estar vacío.")
  validates_presence_of :agente_id, :message => _("Financiador") + _(" no puede estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no puede estar vacía.")
  validates_presence_of :fecha_convocatoria, :message => _("Fecha convocatoria") + " " + _("no puede estar vacía."), :unless => :convenio_id?

  before_validation :copiar_datos_convenio, :on => :create
  after_create :crear_espacio, :copiar_relaciones_convenio
  after_save :nombre_espacio, :cambiar_monedas, :asignar_otras_financiaciones_presupuesto
  after_update :modificar_datos_pacs
  before_destroy :verificar_borrado
  after_destroy :borrar_espacio

  def convenio?
    return self.attributes["convenio_accion"] ? true : false
  end

  # Devuelve el siguiente pac en fechas
  def pac_siguiente
    siguiente = nil
    primera_fecha = self.fecha_de_inicio if self.convenio
    self.convenio.pacs.each do |pac|
      unless pac.etapa.empty?
        fecha_inicio_pac = pac.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.fecha_inicio
        if fecha_inicio_pac > primera_fecha
          siguiente = pac
          primera_fecha = fecha_inicio_pac
        end
      end
    end if self.convenio && primera_fecha
    return siguiente
  end

  # Devuelve el pac anterior en fechas
  def pac_anterior
    anterior = nil
    ultima_fecha = self.convenio.fecha_de_inicio if self.convenio
    self.convenio.pacs.each do |pac|
      unless pac.etapa.empty? || pac == self
        fecha_inicio_pac = pac.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.fecha_inicio 
        if fecha_inicio_pac < self.fecha_de_inicio && fecha_inicio_pac >= ultima_fecha
          anterior = pac
          ultima_fecha = fecha_inicio_pac
        end
      end
    end if self.convenio && ultima_fecha && self.fecha_de_inicio
    return anterior
  end

  def fecha_de_inicio
    (etapa.sort_by {|a| a.fecha_inicio}).first.fecha_inicio unless etapa.empty?
  end

  def fecha_de_fin
    (etapa.sort_by {|a| a.fecha_fin}).last.fecha_fin unless etapa.empty?
  end

  # Si es un pac, copia los datos basicos del convenio antes de guardarse
  def copiar_datos_convenio
    if convenio_id
      self.agente_id = self.convenio.agente_id
      self.moneda_id = self.convenio.moneda_id
      self.moneda_intermedia_id = self.convenio.moneda_intermedia_id
      self.libro_id = self.convenio.libro_id
    end
  end

  # Si es un pac, copia las relaciones del convenio
  def copiar_relaciones_convenio
    if convenio_id
      # Copia los usuarios asignados y sus roles
      self.convenio.usuario_x_proyecto.each { |uxp| self.usuario_x_proyecto.create(:usuario_id => uxp.usuario_id, :rol => uxp.rol) }
      # Copia los elementos del menu de relaciones
      self.convenio.proyecto_x_pais.each { |pxp| self.proyecto_x_pais.create(:pais_id => pxp.pais_id) }
      self.convenio.proyecto_x_moneda.each { |mxp| self.proyecto_x_moneda.find_or_create_by_moneda_id(mxp.moneda_id) }
      self.convenio.proyecto_x_financiador.each { |pxf| self.proyecto_x_financiador.find_or_create_by_agente_id(pxf.agente_id) }
      self.convenio.proyecto_x_implementador.each { |pxi| self.proyecto_x_implementador.find_or_create_by_agente_id(pxi.agente_id) }
      self.convenio.libro_x_proyecto.each { |lxp| self.libro_x_proyecto.find_or_create_by_libro_id(lxp.libro_id) }
      ProyectoXSectorPoblacion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxs|
        ProyectoXSectorPoblacion.create(:proyecto_id => self.id, :sector_poblacion_id => pxs.sector_poblacion_id)
      end
      ProyectoXAreaActuacion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxa|
        ProyectoXAreaActuacion.create(:proyecto_id => self.id, :area_actuacion_id => pxa.area_actuacion_id)
      end
      ProyectoXSectorIntervencion.all(:conditions => {:proyecto_id => self.convenio_id}).each do |pxs|
        ProyectoXSectorIntervencion.create(:proyecto_id => self.id, :sector_intervencion_id => pxs.sector_intervencion_id)
      end

      # Copia las partidas y sus relaciones
      self.convenio.partida_financiacion.each do |pf|
        #mipf = self.partida_financiacion.create(pf.attributes)
        mipf = pf.clone
        mipf.proyecto_id = self.id
        mipf.save
        pf.partida_x_partida_financiacion.each { |pxpf| mipf.partida_x_partida_financiacion.create(:partida_id => pxpf.partida_id) }
      end

      # Copia las subpartidas y sus relaciones
      self.convenio.subpartida.each do |sp|
        misp = sp.clone
        misp.proyecto_id = self.id
        misp.save
      end

      # Copia los objetivos especificos, resultados y actividades (junto a las hipotesis, fuentes e indicadores)
      self.convenio.objetivo_especifico.each do |obj|
        obj.crear_asociacion_pacs
        obj.fuente_verificacion.each{|fv| fv.crear_asociacion_pacs}
        obj.indicador.each{|ind| ind.crear_asociacion_pacs}
        obj.hipotesis.each{|hip| hip.crear_asociacion_pacs}
      end
      self.convenio.resultado.each do |obj|
        obj.crear_asociacion_pacs
        obj.fuente_verificacion.each{|fv| fv.crear_asociacion_pacs}
        obj.indicador.each{|ind| ind.crear_asociacion_pacs}
        obj.hipotesis.each{|hip| hip.crear_asociacion_pacs}
      end
      self.convenio.actividad.each do |obj|
        obj.crear_asociacion_pacs
      end
    end
  end

  # Si es un convenio, copia los datos a sus pacs despues de una modificacion
  def modificar_datos_pacs
    self.pacs.each do |pac|
      pac.agente_id = self.agente_id
      pac.moneda_id = self.moneda_id
      pac.moneda_intermedia_id = self.moneda_intermedia_id
      pac.libro_id = self.libro_id
      pac.save      
    end if self.convenio?
  end

  def crear_espacio
    espacio_padre = Espacio.find_by_nombre("Proyectos").id unless self.convenio
    espacio_padre = Espacio.find_by_nombre(self.convenio.nombre).id if self.convenio
    espacio_proyecto = Espacio.create :nombre => nombre, :proyecto_id => id, :espacio_padre_id => espacio_padre, :descripcion => ("Espacio raiz del proyecto: " + nombre)
    for espacio in Espacio.find_all_by_definicion_espacio_proyecto(true)
      Espacio.create :nombre => espacio.nombre, :espacio_padre_id => espacio_proyecto.id, :definicion_espacio_proyecto_id => espacio.id
    end
  end

  def borrar_espacio
    for un_espacio in Espacio.find_all_by_definicion_espacio_proyecto(true)
      Espacio.find_by_espacio_padre_id_and_definicion_espacio_proyecto_id(espacio.id, un_espacio.id).destroy
    end
    espacio.destroy
  end



  def nombre_espacio
    espacio.update_attribute "nombre", nombre
  end

  def asignar_otras_financiaciones_presupuesto
    financiacion_privada = Agente.find_by_nombre(_("OTRAS FINANCIACIONES PRIVADAS"))
    financiacion_publica = Agente.find_by_nombre(_("OTRAS FINANCIACIONES PÚBLICAS"))
    ProyectoXFinanciador.find_or_create_by_agente_id_and_proyecto_id(financiacion_privada.id, id) if financiacion_privada
    ProyectoXFinanciador.find_or_create_by_agente_id_and_proyecto_id(financiacion_publica.id, id) if financiacion_publica
  end

  def dato_dinamico id , tipo 
    eval( "Dato" + tipo ).find(id) unless id == 0
    eval( "Dato" + tipo ).new if id == 0 
  end

  def datos_dinamicos pestana
    # ( dato_integro.select{|d| d.definicion_dato.pestana == pestana } + dato_cadena.select{|d| d.definicion_dato.pestana == pestana }  + dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
    ( dato_texto.select{|d| d.definicion_dato.pestana == pestana } ).sort{ |a,b| a.definicion_dato.rango <=> b.definicion_dato.rango }
  end

  def nombre_mayusculas
    self.nombre = self.nombre.upcase
  end

  def estado_actual
    (estado.select { |a|  a.estado_actual }).first
  end

 # Metodos para sustituir algunas de las opciones de presupuesto_total para mejorar el rendimiento
 # pendiente de desarrollar todas las opciones!
 def presupuesto_total_sin_financiador
   resultado = Presupuesto.find_by_sql "select sum(presupuesto.importe * tasa_cambio) as total from presupuesto left join tasa_cambio on tasa_cambio.id = presupuesto.tasa_cambio_id where presupuesto.proyecto_id =  #{self.id}"
   return resultado.first.total.to_f
 end

 def presupuesto_total_con_financiador financiador
   resultado = Presupuesto.find_by_sql "select sum(presupuesto_x_agente.importe * tasa_cambio) as total from presupuesto left join tasa_cambio on tasa_cambio.id = presupuesto.tasa_cambio_id left join presupuesto_x_agente on presupuesto_x_agente.presupuesto_id = presupuesto.id where presupuesto.proyecto_id =#{self.id} and presupuesto_x_agente.agente_id = #{financiador.id};"
   return resultado.first.total.to_f
 end


 # Devuelve el presupuesto total del proyecto en la moneda base o el de la moneda que se especifique
 def presupuesto_total mon=nil, financ=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   condiciones = {} 
   #fecha_inicio = self.etapa.sort{ |a,b| a.fecha_inicio <=> b.fecha_inicio }.first.fecha_inicio if fecha_fin
   #condiciones = { :fecha => fecha_inicio..fecha_fin } if fecha_fin && fecha_fin.class.name == "Date"
   condiciones["presupuesto_x_agente.agente_id"] = financ.id if financ
   condiciones["presupuesto.moneda_id"] = mon.id if mon and mon.class.name=="Moneda"
   self.presupuesto.all(:include => "presupuesto_x_agente", :conditions => condiciones).each do |pre|
     importe = financ ? pre.presupuesto_x_agente.first.importe : pre.importe
     total += importe if mon
     total += (importe * pre.tasa_cambio).round(2) unless mon
   end
   return total
 end

 # Metodos para sustituir algunas de las opciones de gasto_total para mejorar el rendimiento
 # pendiente de desarrollar todas las opciones!
 def gasto_total_sin_financiador
   # Si es un convenio, devuelve la suma de gasto de los pacs
   proyectos = self.convenio_accion ? self.pacs : [self]
   total = 0
   proyectos.each do |p|
     resultado = Gasto.find_by_sql "select sum(gasto_x_proyecto.importe * tasa_cambio) as total from gasto left join gasto_x_proyecto on gasto.id = gasto_x_proyecto.gasto_id left join tasa_cambio on tasa_cambio.id = gasto_x_proyecto.tasa_cambio_id where  gasto_x_proyecto.proyecto_id = #{p.id}"
     total += resultado.first.total.to_f
   end
   return total 
 end

 def gasto_total_con_financiador financiador
   # Si es un convenio, devuelve la suma de gasto de los pacs
   proyectos = self.convenio_accion ? self.pacs : [self]
   total = 0
   proyectos.each do |p|
     resultado = Gasto.find_by_sql "  select sum(gasto_x_agente.importe * tasa_cambio) as total from gasto left join gasto_x_proyecto on gasto.id = gasto_x_proyecto.gasto_id left join tasa_cambio on tasa_cambio.id = gasto_x_proyecto.tasa_cambio_id left join gasto_x_agente on gasto_x_agente.gasto_id = gasto.id and gasto_x_agente.proyecto_id = gasto_x_proyecto.proyecto_id where  gasto_x_proyecto.proyecto_id = #{p.id} and gasto_x_agente.agente_id = #{financiador.id} and gasto_x_agente.proyecto_id = #{p.id} "
     total += resultado.first.total.to_f
   end
   return total
 end
  



 # Devuelve el gasto total del proyecto en la indicada o la moneda base si no esta definida
 def gasto_total financ=nil, mon=nil, etap=nil, pai=nil
   total = 0.0
   financ = nil unless financ.class.name == "Agente"
   etap = nil unless etap.class.name == "Etapa"
   mon = nil unless mon.class.name == "Moneda"
   pai = nil unless pai.class.name == "Pais"
   condiciones = {}
   condiciones["gasto_x_agente.agente_id"] = financ.id if financ
   condiciones["gasto.fecha"] = etap.fecha_inicio..etap.fecha_fin if etap
   condiciones["gasto.moneda_id"] = mon.id if mon
   # Recorremos todos los gastos
   self.gasto.all(:include => ["gasto_x_agente", "pago"], :conditions => condiciones).each do |gas|
     # PENDIENTE: Tenemos que filtrar solo los gastos pagados con una cuenta del pais indicado (si existe)
     importe_pago = 0
     gas.pago.each do |pago|
       importe_pago += pago.importe if pago.libro && pago.libro.pais && pago.libro.pais == pai 
     end
     # Pillamos el gasto por financiador si se ha pedido eso (financ != nil)
     gxa = gas.gasto_x_agente.first if financ
     # Pillamos el gasto por proyecto (necesario para la tasa de cambio)
     gxp = gas.gasto_x_proyecto.find_by_proyecto_id self.id
     # Escogemos el importe segun sea de financiador o completo del proyecto
     importe = gxa ? gxa.importe : gxp.importe

     # Averiguamos el importe de los pagos realizados (y en el caso de que hayamos definido pais, sobre cuenta del pais)
     importe_pagado = 0
     gas.pago.each { |pago| importe_pagado += pago.importe if pai.nil? || pago.libro.pais == pai }

     # Y escogemos el adecuado (puede que no este todo pagado, o que los pagos sean mayores que el importe)
     importe = [importe, importe_pagado].min if importe >= 0
     importe = [importe, importe_pagado].max if importe < 0

     # Si no hay moneda, le aplicamos la tasa de cambio
     total += (importe * gxp.tasa_cambio).round(2) if importe && gxp && mon.nil?
     # Si hay moneda, no convertimos el importe 
     total += importe if importe && mon
   end
   #puts "----------------> Nos piden el gasto para"
   #puts "                  PAIS: " + pai.nombre if pai
   #puts "                  MONEDA: " + mon.nombre if mon
   #puts "                  ETAPA: " + etap.nombre if etap
   #puts "                  FINANC: " + financ.nombre if financ
   #puts "                  " + total.to_s
   return total
 end

 # Devuelve el total de transferencias realizadas (para una o todas las monedas)
 def transferido_total mon, etap=nil, financ=nil, direccion="entrante", tipo=nil, pai=nil
   total = 0.0
   pai = nil unless pai.class.name == "Pais"
   if mon && mon.class.name == "Moneda"

     incluye_tabla = "libro_origen" if direccion == "saliente"
     incluye_tabla = "libro_destino" if direccion == "entrante"
    
     # Filtros basicos de la transferencia... luego los complicaremos sobre los resultados
     #condiciones  = "moneda_id = " + mon.id.to_s 
     condiciones = "libro.moneda_id = " + mon.id.to_s
     condiciones += " AND transferencia.tipo = '" + tipo.to_s + "'" if tipo
     condiciones += " AND transferencia_x_agente.agente_id = " + financ.id.to_s if financ.class.name == "Agente"
     if etap.class.name == "Etapa"
       condiciones += " AND " + (direccion == "saliente" ? "fecha_enviado" : "fecha_recibido") + " BETWEEN '" + etap.fecha_inicio.to_s + "' AND '" + etap.fecha_fin.to_s + "'" 
     end
    
     logger.info "---------------------> Miramos sobre la moneda " + mon.abreviatura + " con las condiciones " + condiciones
     logger.info "---------------------> Incluyendo la tabla: " + incluye_tabla
     logger.info "---------------------> Pais: " + pai.nombre if pai
     transf_filtradas = self.transferencia.all(:include => [incluye_tabla, 'transferencia_x_agente'], :conditions => condiciones)
     logger.info "---------------------> " + transf_filtradas.inspect
     transf_filtradas.each do |trans|
       # Contabiliza solo las que sean a distinta moneda o distinto pais a no ser que sea sin segunda cuenta 
       if (tipo && trans.tipo =~ /subvencion|intereses|adelanto|devolucion/) || (trans.libro_origen && trans.libro_destino && (trans.libro_origen.moneda_id != trans.libro_destino.moneda_id || trans.libro_origen.pais_id != trans.libro_destino.pais_id))
         # Contabiliza solo las transferencias del pais (o si no se pide por pais, todas)
         if pai == nil ||
		(direccion == "saliente" && trans.libro_origen && trans.libro_origen.pais == pai) ||
		(direccion == "entrante" && trans.libro_destino && trans.libro_destino.pais == pai)
           # Selecciona el importe total ...
           unless financ.class.name == "Agente"
             importe_tmp = (direccion == "saliente" ? trans.importe_enviado : trans.importe_cambiado)
           # ... o el del financiador (aqui hay que ajustar para que coja el mismo porcentaje en el caso de que sea entrante)
           else 
             txf = trans.transferencia_x_agente.find_by_agente_id(financ.id)
            
             # Para salientes si existe transferencia_x_financiador para ese financiador 
             importe_tmp = txf.importe if direccion == "saliente" && txf
            
             # Y para entrantes, si existe transferencia_x_financiador para ese financiador y la transferencia se ha recibido, calcula el porcentaje sobre el enviado o el recibido
             if direccion == "entrante" && trans.importe_cambiado && txf
               logger.info "Tenemos importe_cambiado: " + trans.importe_cambiado.inspect + " importe_enviado: " + trans.importe_enviado.inspect + " y txf " + txf.inspect
               importe_tmp = trans.importe_cambiado * (txf.importe / (trans.importe_enviado || trans.importe_recibido))
             end
           end
           # Si se ha indicado la direccion, se contabilizan solo esas
           total += importe_tmp if importe_tmp
         end
       end
     end if transf_filtradas
   end
   return total
 end

	# Devuelve el resumen de tesoreria para un financiador y una etapa dada
  def tesoreria financ, etap, mon_base=true
    filas = []
    cabecera_fila = []
    paises = self.pais

    # Genera la cabecera de las columnas a mostrar...
    # ... base sin pais
    cabecera_col = [ {:pais_id => nil, :moneda_id => self.moneda_principal.id} ]
    # ... moneda principal por paises
    cabecera_col += paises.collect { |p| {:pais_id => p.id, :moneda_id => self.moneda_principal.id} unless p.moneda.find_by_id(self.moneda_id) }
    # ... divisa por paises
    cabecera_col += paises.collect { |p| {:pais_id => p.id, :moneda_id => self.moneda_intermedia.id} unless self.moneda_intermedia.nil? || p.moneda.find_by_id(self.moneda_intermedia_id) }
    # ... monedas locales por paises
    paises.each { |p| cabecera_col += p.moneda.collect{|m| {:pais_id => p.id, :moneda_id => m.id} } }
    # ... y limpia los nil que haya en el array
    cabecera_col.delete(nil)

    # Obtiene el saldo anterior del pac previo
    saldo_anterior = nil
    if self.pac_anterior
      # A revisar esta invocacion segun quede
      (prev_cab_col,prev_cab_fil,prev_filas) = self.pac_anterior.tesoreria(financ,etap,mon_base)
      saldo_anterior_euros = prev_filas[-1][0] 
      saldo_anterior = prev_filas[-4]
      cambio_anterior = prev_filas[-3]
    end

    # Transferencias de subvenciones (por agente y total) -> Habra problemas con financiaciones en otras monedas!!!
    principal_financia = self.transferido_total(self.moneda_principal, etap, financ, "entrante", "subvencion", nil)
    total_financia = self.transferido_total(self.moneda_principal, etap, nil, "entrante", "subvencion", nil)

    # Dinero transferido a los paises (entrante y saliente) para moneda del proyecto e intermedia (evitando los paises que las tengan como local)
    principal_entrante = paises.collect do |p|
      self.transferido_total(self.moneda_principal, etap, financ, "entrante", nil, p) unless p.moneda.find_by_id(self.moneda_id)
    end
    principal_saliente = paises.collect do |p|
      self.transferido_total(self.moneda_principal, etap, financ, "saliente", nil, p) unless p.moneda.find_by_id(self.moneda_id)
    end
    divisa_saliente = []
    divisa_saliente = paises.collect do |p|
      self.transferido_total(self.moneda_intermedia, etap, financ, "saliente", nil, p) unless p.moneda.find_by_id(self.moneda_intermedia_id)
    end if self.moneda_intermedia
    divisa_entrante = []
    divisa_entrante = paises.collect do |p|
      self.transferido_total(self.moneda_intermedia, etap, financ, "entrante", nil, p) unless p.moneda.find_by_id(self.moneda_intermedia_id)
    end if self.moneda_intermedia
    principal_saliente.delete(nil)
    principal_entrante.delete(nil)
    divisa_saliente.delete(nil)
    divisa_entrante.delete(nil)
    divisa_total_entrante = principal_entrante + divisa_entrante
    divisa_total_saliente = principal_saliente + divisa_saliente
   
    # Moneda local recibida
    locales_entrante = paises.inject([]){ |sum, p| sum + p.moneda.collect{|m| self.transferido_total(m, etap, financ, "entrante", nil, p)} }
    # Moneda local saliente
    locales_saliente = paises.inject([]){ |sum, p| sum + p.moneda.collect{|m| self.transferido_total(m, etap, financ, "saliente", nil, p)} }

    # Transferido desde la moneda principal menos las trasferencias dentro de otros paises como divisa
    principal_transferido  = principal_saliente.inject(self.transferido_total(self.moneda_principal, etap, financ, "saliente")){|sum,p| sum - p}
    #  o como moneda local igual a la principal
    paises.each do |p|
      principal_transferido -= self.transferido_total(self.moneda_principal, etap, financ, "saliente", nil, p) if p.moneda.find_by_id(self.moneda_principal.id)
    end

    # Intereses a las cuentas en moneda principal en los paises
    intereses_principal_paises = paises.collect do |p|
      self.transferido_total(self.moneda_principal, etap, nil, "entrante", "intereses", p) unless p.moneda.find_by_id(self.moneda_id)
    end
    intereses_principal_paises.delete(nil)
    # Resto de intereses de la moneda principal
    intereses_principal = self.transferido_total(self.moneda_principal, etap, nil, "entrante", "intereses", nil)
    intereses_principal_paises.each{|i| intereses_principal -= i}
    # Quitando tambien las de las monedas locales igual a la principal (esto falta)
    paises.each do |p|
      intereses_principal -= self.transferido_total(self.moneda_principal, etap, nil, "entrante", "intereses", p) if p.moneda.find_by_id(self.moneda_principal.id)
    end

    # Intereses de la moneda principal
    intereses = [ intereses_principal ] + intereses_principal_paises
    # Intereses para divisa
    intereses += paises.collect do |p|
      self.transferido_total(self.moneda_intermedia, etap, nil, "entrante", "intereses", p) unless p.moneda.find_by_id(self.moneda_intermedia_id)
    end if self.moneda_intermedia
    intereses.delete(nil)
    # Intereses para monedas locales
    paises.each{|p| intereses += p.moneda.collect{|m| self.transferido_total(m, etap, nil, "entrante", "intereses", p) } }

    porcentaje_intereses = total_financia == 0 ? 0 : principal_financia / total_financia
    intereses = intereses.collect { |v| v * porcentaje_intereses }


    #  Fondos totales de la subvencion
    total_fondos = [  principal_financia - principal_transferido] +
                      divisa_total_entrante.zip(divisa_total_saliente).map {|a| a.inject(:-)} +
                      locales_entrante.zip(locales_saliente).map {|a| a.inject(:-)}
    # ... incluyendo remanentes
    total_fondos = total_fondos.zip(saldo_anterior).map {|a| a.inject(:+)} if saldo_anterior && !mon_base
    total_fondos[0] += saldo_anterior_euros if saldo_anterior && mon_base
    # ... e incluyendo intereses
    total_fondos = total_fondos.zip(intereses).map {|a| a.inject(:+)}

    # Gastos
    #  gastos en la moneda principal como divisa por paises
    gastos_principal = paises.collect do |p|
      self.gasto_total(financ, self.moneda_principal, etap, p) unless p.moneda.find_by_id(self.moneda_id)
    end
    gastos_principal.delete(nil)

    #  y el resto de gastos de la moneda principal (el total menos en divisa)
    gastos_principal_local = gastos_principal.inject(self.gasto_total(financ, self.moneda_principal, etap, nil)) { |sum,g| sum - g }
    #  quitando tambien los gastos de paises con moneda local como principal (esto falta)
    paises.each do |p|
      gastos_principal_local -= self.gasto_total(financ, self.moneda_principal, etap, p) if p.moneda.find_by_id(self.moneda_principal.id)
    end

    # El conjunto total de gastos: Al principal local + principal como divisa 
    #   le incluimos las divisas
    gastos  = [gastos_principal_local] + gastos_principal
    gastos += paises.collect do |p|
      self.gasto_total(financ, self.moneda_intermedia, etap, p) unless p.moneda.find_by_id(self.moneda_intermedia_id)
    end if self.moneda_intermedia
    #   y las monedas de cada pais
    paises.each{|p| gastos += p.moneda.collect{|m| self.gasto_total(financ, m, etap, p)} }
    gastos.delete(nil)

    # Saldos corrientes
    saldos = total_fondos.zip(gastos).map {|a| a.inject(:-)}

    # Tasas de Cambio
    # Hay que corregir la ultima fecha para que sea de TC y no de etapa 
    ultima_fecha = etap ? etap.fecha_fin : (self.etapa.empty? ? Date.new : self.etapa.last(:order => "fecha_fin").fecha_fin)

    #  tasa de cambio para moneda principal y divisa (si existe)
    tasa_cambio = [ 1 ] + principal_saliente.collect{|a| 1}
    if self.moneda_intermedia_id
      gasto_falso_divisa = Gasto.new(:moneda_id => self.moneda_intermedia_id, :fecha => ultima_fecha)
      tc = TasaCambio.tasa_cambio_para_gasto(gasto_falso_divisa, self)
      tc_valor = tc ? tc.tasa_cambio : 0
      tasa_cambio += divisa_saliente.collect{|a| tc_valor }
    end

    #  tasa de cambio por paises y monedas
    paises.each { |p| tasa_cambio += p.moneda.collect{|m|
      tcg = TasaCambio.tasa_cambio_para_gasto(Gasto.new(:moneda_id => m.id, :fecha => ultima_fecha), self)
      tcg ? tcg.tasa_cambio : 0
    } }

    # Totales
    saldos_convertidos = saldos.zip(tasa_cambio).map {|a| a.inject(:*)}
    total_euros = saldos_convertidos.inject(0){|sum, sc| sum + sc}

    # Construye la salida final
    if saldo_anterior
      cabecera_fila.push( _("(+) Saldos de periodos anteriores") )
      filas.push( [saldo_anterior_euros] ) if mon_base
      filas.push( saldo_anterior ) unless mon_base
    end
    cabecera_fila.push( _("(+) Subvención recibida ") + financ.nombre )
    filas.push( [ principal_financia] )
    cabecera_fila.push( _("(+) Total Divisa intermedia") )
    filas.push( [""] + divisa_total_entrante )
    cabecera_fila.push( _("(-) Total Divisa recibida y cambiada") )
    filas.push( [""] + divisa_total_saliente )
    cabecera_fila.push( _("(+) Total Moneda Local obtenida") )
    filas.push( [""] + divisa_total_saliente.collect{|a| ""} + locales_entrante )
    cabecera_fila.push( _("(-) Moneda Local recibida y cambiada") )
    filas.push( [""] + divisa_total_saliente.collect{|a| ""} + locales_saliente )
    cabecera_fila.push( _("(+) TOTAL INTERESES GENERADOS") )
    filas.push( intereses )
    cabecera_fila.push( _("(I) TOTAL Fondos de la subvencion") )
    filas.push( total_fondos )
    cabecera_fila.push( _("(II) GASTOS aplicados (-)") )
    filas.push( gastos )
    cabecera_fila.push( _("(III) SALDOS CORRIENTES (I) - (II)") )
    filas.push( saldos )
    cabecera_fila.push( _("Tipo de cambio aplicable") )
    filas.push( tasa_cambio )
    cabecera_fila.push( _("(IV) SALDO") + " " + self.moneda_principal.nombre.pluralize + " " + _("EQUIVALENTES") )
    filas.push( saldos_convertidos )
    cabecera_fila.push( _("(V) TOTAL") + " " + self.moneda_principal.nombre.pluralize )
    filas.push( [total_euros] )

    return cabecera_col, cabecera_fila, filas 
  end

	# Aplica los remanentes segun el estado de tesoreria del PAC anterior
  def aplica_remanentes mon_base=true
    # Borra todos los remanentes existentes
    Transferencia.all(:conditions => {:remanente => true, :proyecto_id => self.id}).each {|rem| rem.destroy} if self.pac_anterior
    # Si hay un pac anterior recorre todas las cuentas
    self.libro.each do |l|
      # Si el libro es el principal del proyecto pone el total
      if self.libro_id == l.id
        (cab_col, cab_fila, filas) = self.pac_anterior.tesoreria(self.agente, nil)
        rem_importe = filas[-1][0]
      # Para el resto de los libros...  
      else
        # ... obtiene el arqueo de caja del libro (sobre el financiador principal)
        arqueo =  l.arqueo [ self.pac_anterior ], nil, nil, self.agente
        rem_importe = arqueo[:entrante] - arqueo[:saliente]
      end
      # Si el remanente es positivo, se incluye
      if (rem_importe) != 0.0
        gasto_falso = Gasto.new(:moneda_id => l.moneda_id, :fecha => self.pac_anterior.fecha_de_fin)
        # Obtiene su ultima tasa de cambio
        tc = TasaCambio.tasa_cambio_para_gasto(gasto_falso, self.pac_anterior)
        tc_valor = tc ? tc.tasa_cambio : nil
        # Genera una transferencia con el remanente (con new para incluir callbacks)
        t = Transferencia.new(	:proyecto_id => self.id, :remanente => true, :tipo => "remanente",
				:libro_origen_id => self.libro_id, :libro_destino_id => l.id,
				:fecha_recibido => self.fecha_de_inicio,
				:importe_recibido => rem_importe * tc_valor, :importe_cambiado => rem_importe, :tasa_cambio => tc_valor,
				:observaciones => _("Remanente") + " " + self.pac_anterior.nombre )
	# Si los libros no son los mismos, introduce los datos de enviado
        if self.libro_id != l.id
          t.importe_enviado = t.importe_recibido 
          t.fecha_enviado = self.fecha_de_inicio 
        end
        t.save
        # Y si todo esta bien guardado, el financiador principal
        txf = TransferenciaXAgente.create( :transferencia_id => t.id, :agente_id => self.agente_id, :importe => t.importe_recibido) if t.errors.empty?
      end
    end if self.pac_anterior
  end

  def presupuesto_cofinanciado_privado
    
  end

  private

  def cambiar_monedas
    # Se asegura de que la moneda principal este asignada
    ProyectoXMoneda.find_or_create_by_proyecto_id_and_moneda_id(self.id, self.moneda_id) if moneda_id_changed? && moneda_id
    # Y tambien la divisa
    ProyectoXMoneda.find_or_create_by_proyecto_id_and_moneda_id(self.id, self.moneda_intermedia_id) if moneda_intermedia_id_changed? && moneda_intermedia_id

    # Para modificaciones de proyecto que incluyan moneda principal, actualiza las tasas de cambio
    self.etapa.each { |e| TasaCambio.cambia_moneda(e) } if id_was && moneda_id_changed?
  end

  def verificar_borrado
    #if self.datos_proyecto
    #  (DatosProyecto.column_names - ["id", "proyecto_id"]).each do |v|
    #    if not self.datos_proyecto.send( v.to_sym ).nil?
    #      errors.add( "datos_proyecto","hay datos de proyecto " + v.humanize )
    #    end
    #  end
    #end
    errors.add( "gasto", "hay gastos" ) unless self.gasto_x_proyecto.empty?
    errors.add( "presupuesto", "hay presupuestos" ) unless self.presupuesto.empty?
    errors.add( "objetivo_especifico", "hay objetivos especificos" ) unless self.objetivo_especifico.empty?
    errors.add( "documento", "hay documentos" ) unless self.documento.empty?
    #errors.add( "tarea", "hay tareas" ) unless self.tarea.empty?
    # la relacion habtm no tiene :dependent por eso tenemos que borrar las relaciones de financiador y implementador 
    #errors.add( _("esta relacionado con agentes como implementadores") ) unless self.implementador == []
    #errors.add( _("esta relacionado con agentes como financiadores") ) unless self.financiador == []
    errors.add_to_base( "Un proyecto tiene que estar vacio para poder ser borrado." ) unless errors.empty?
   return false unless errors.empty?
 end

 #def pais_principal
 # return  (pais_principal_id ? Pais.find_by_pais_principal_id(pais_principal_id).nombre : "")
 #end

end
